[llvm] Debug names spec abstract origin (PR #85485)
Alexander Yermolovich via llvm-commits
llvm-commits at lists.llvm.org
Fri Mar 15 16:49:53 PDT 2024
https://github.com/ayermolo created https://github.com/llvm/llvm-project/pull/85485
None
>From 708d2a09fb0f4585ec190d105a8b03db2e86b892 Mon Sep 17 00:00:00 2001
From: Alexander Yermolovich <ayermolo at meta.com>
Date: Wed, 13 Mar 2024 17:36:55 -0700
Subject: [PATCH 1/2] [BOLT][DWARF] Add support for DW_IDX_parent
Summary:
This adds support for DW_IDX_parent. If DIE has a parent then DW_IDX_parent in Entry
will point to Entry for that parent DIE. Otherwise it will have DW_FORM_flag_present in
abbrev. Which takes zero space in Entry.
This came from
https://discourse.llvm.org/t/rfc-improve-dwarf-5-debug-names-type-lookup-parsing-speed/74151
Test Plan: ninja check-bolt
Reviewers: maks, davidino, #llvm-bolt
Reviewed By: maks
Differential Revision: https://phabricator.intern.facebook.com/D54882962
Tasks: T181795932
---
bolt/include/bolt/Core/DIEBuilder.h | 8 +-
bolt/include/bolt/Core/DebugNames.h | 17 +-
bolt/lib/Core/DIEBuilder.cpp | 28 +-
bolt/lib/Core/DebugNames.cpp | 83 +++++-
...arf5-debug-names-generate-debug-names.test | 16 +-
bolt/test/X86/dwarf5-debug-names.test | 38 ++-
...5-df-debug-names-generate-debug-names.test | 19 +-
bolt/test/X86/dwarf5-df-debug-names.test | 278 +++++++++---------
...warf5-df-main-debug-names-ftu-ltu-mix.test | 3 +
.../X86/dwarf5-df-one-cu-debug-names.test | 12 +-
.../test/X86/dwarf5-df-types-debug-names.test | 24 +-
.../dwarf5-df-types-one-cu-debug-names.test | 15 +-
bolt/test/X86/dwarf5-one-cu-debug-names.test | 42 ++-
bolt/test/X86/dwarf5-types-debug-names.test | 14 +-
.../X86/dwarf5-types-one-cu-debug-names.test | 12 +-
15 files changed, 424 insertions(+), 185 deletions(-)
diff --git a/bolt/include/bolt/Core/DIEBuilder.h b/bolt/include/bolt/Core/DIEBuilder.h
index 4debf936335472..06084819ec0b3a 100644
--- a/bolt/include/bolt/Core/DIEBuilder.h
+++ b/bolt/include/bolt/Core/DIEBuilder.h
@@ -209,7 +209,13 @@ class DIEBuilder {
void updateReferences();
/// Update the Offset and Size of DIE, populate DebugNames table.
- uint32_t finalizeDIEs(DWARFUnit &CU, DIE &Die, uint32_t &CurOffset);
+ /// Along with current CU, and DIE being processed and the new DIE offset to
+ /// be updated, it takes in Parents vector that can be empty if this DIE has
+ /// no parents.
+ uint32_t
+ finalizeDIEs(DWARFUnit &CU, DIE &Die,
+ std::vector<std::optional<BOLTDWARF5AccelTableData *>> &Parents,
+ uint32_t &CurOffset);
void registerUnit(DWARFUnit &DU, bool NeedSort);
diff --git a/bolt/lib/Core/DIEBuilder.cpp b/bolt/lib/Core/DIEBuilder.cpp
index 42287978a8576f..0cf8a5e8c2c3db 100644
--- a/bolt/lib/Core/DIEBuilder.cpp
+++ b/bolt/lib/Core/DIEBuilder.cpp
@@ -377,20 +377,32 @@ getUnitForOffset(DIEBuilder &Builder, DWARFContext &DWCtx,
return nullptr;
}
-uint32_t DIEBuilder::finalizeDIEs(DWARFUnit &CU, DIE &Die,
- uint32_t &CurOffset) {
+uint32_t DIEBuilder::finalizeDIEs(
+ DWARFUnit &CU, DIE &Die,
+ std::vector<std::optional<BOLTDWARF5AccelTableData *>> &Parents,
+ uint32_t &CurOffset) {
getState().DWARFDieAddressesParsed.erase(Die.getOffset());
uint32_t CurSize = 0;
Die.setOffset(CurOffset);
- DebugNamesTable.addAccelTableEntry(
- CU, Die, SkeletonCU ? SkeletonCU->getDWOId() : std::nullopt);
+ std::optional<BOLTDWARF5AccelTableData *> NameEntry =
+ DebugNamesTable.addAccelTableEntry(
+ CU, Die, SkeletonCU ? SkeletonCU->getDWOId() : std::nullopt,
+ Parents.back());
+ // It is possible that an indexed debugging information entry has a parent
+ // that is not indexed (for example, if its parent does not have a name
+ // attribute). In such a case, a parent attribute may point to a nameless
+ // index entry (that is, one that cannot be reached from any entry in the name
+ // table), or it may point to the nearest ancestor that does have an index
+ // entry.
+ if (NameEntry)
+ Parents.push_back(std::move(NameEntry));
for (DIEValue &Val : Die.values())
CurSize += Val.sizeOf(CU.getFormParams());
CurSize += getULEB128Size(Die.getAbbrevNumber());
CurOffset += CurSize;
for (DIE &Child : Die.children()) {
- uint32_t ChildSize = finalizeDIEs(CU, Child, CurOffset);
+ uint32_t ChildSize = finalizeDIEs(CU, Child, Parents, CurOffset);
CurSize += ChildSize;
}
// for children end mark.
@@ -400,6 +412,8 @@ uint32_t DIEBuilder::finalizeDIEs(DWARFUnit &CU, DIE &Die,
}
Die.setSize(CurSize);
+ if (NameEntry)
+ Parents.pop_back();
return CurSize;
}
@@ -410,7 +424,9 @@ void DIEBuilder::finish() {
uint32_t HeaderSize = CU.getHeaderSize();
uint32_t CurOffset = HeaderSize;
DebugNamesTable.setCurrentUnit(CU, UnitStartOffset);
- finalizeDIEs(CU, *UnitDIE, CurOffset);
+ std::vector<std::optional<BOLTDWARF5AccelTableData *>> Parents;
+ Parents.push_back(std::nullopt);
+ finalizeDIEs(CU, *UnitDIE, Parents, CurOffset);
DWARFUnitInfo &CurUnitInfo = getUnitInfoByDwarfUnit(CU);
CurUnitInfo.UnitOffset = UnitStartOffset;
diff --git a/bolt/lib/Core/DebugNames.cpp b/bolt/lib/Core/DebugNames.cpp
index 384e63695dfdc5..4bcedbea8d2252 100644
--- a/bolt/lib/Core/DebugNames.cpp
+++ b/bolt/lib/Core/DebugNames.cpp
@@ -13,6 +13,7 @@
#include "llvm/Support/EndianStream.h"
#include "llvm/Support/LEB128.h"
#include <cstdint>
+#include <optional>
namespace llvm {
namespace bolt {
@@ -163,10 +164,16 @@ static uint64_t getNameOffset(BinaryContext &BC, DWARFUnit &Unit,
Index * DwarfOffsetByteSize);
}
-void DWARF5AcceleratorTable::addAccelTableEntry(
- DWARFUnit &Unit, const DIE &Die, const std::optional<uint64_t> &DWOID) {
+static uint64_t getEntryID(const BOLTDWARF5AccelTableData &Entry) {
+ return reinterpret_cast<uint64_t>(&Entry);
+}
+
+std::optional<BOLTDWARF5AccelTableData *>
+DWARF5AcceleratorTable::addAccelTableEntry(
+ DWARFUnit &Unit, const DIE &Die, const std::optional<uint64_t> &DWOID,
+ std::optional<BOLTDWARF5AccelTableData *> &Parent) {
if (Unit.getVersion() < 5 || !NeedToCreate)
- return;
+ return std::nullopt;
std::string NameToUse = "";
auto canProcess = [&](const DIE &Die) -> bool {
switch (Die.getTag()) {
@@ -217,7 +224,7 @@ void DWARF5AcceleratorTable::addAccelTableEntry(
};
if (!canProcess(Die))
- return;
+ return std::nullopt;
// Addes a Unit to either CU, LocalTU or ForeignTU list the first time we
// encounter it.
@@ -227,10 +234,11 @@ void DWARF5AcceleratorTable::addAccelTableEntry(
addUnit(Unit, DWOID);
}
- auto addEntry = [&](DIEValue ValName) -> void {
+ auto addEntry =
+ [&](DIEValue ValName) -> std::optional<BOLTDWARF5AccelTableData *> {
if ((!ValName || ValName.getForm() == dwarf::DW_FORM_string) &&
NameToUse.empty())
- return;
+ return std::nullopt;
std::string Name = "";
uint64_t NameIndexOffset = 0;
if (NameToUse.empty()) {
@@ -275,13 +283,23 @@ void DWARF5AcceleratorTable::addAccelTableEntry(
<< ".\n";
SecondIndex = Iter->second;
}
+ std::optional<uint64_t> ParentOffset =
+ (Parent ? std::optional<uint64_t>(getEntryID(**Parent)) : std::nullopt);
+ // This will be populated later in writeEntry.
+ // This way only parent entries get tracked.
+ // Keeping memory footprint down.
+ if (ParentOffset)
+ EntryRelativeOffsets.insert({*ParentOffset, 0});
It.Values.push_back(new (Allocator) BOLTDWARF5AccelTableData(
- Die.getOffset(), std::nullopt, DieTag, UnitID, IsTU, SecondIndex));
+ Die.getOffset(), ParentOffset, DieTag, UnitID, IsTU, SecondIndex));
+ return It.Values.back();
};
- addEntry(Die.findAttribute(dwarf::Attribute::DW_AT_name));
- addEntry(Die.findAttribute(dwarf::Attribute::DW_AT_linkage_name));
- return;
+ std::optional<BOLTDWARF5AccelTableData *> NameEntry =
+ addEntry(Die.findAttribute(dwarf::Attribute::DW_AT_name));
+ std::optional<BOLTDWARF5AccelTableData *> LinkageNameEntry =
+ addEntry(Die.findAttribute(dwarf::Attribute::DW_AT_linkage_name));
+ return NameEntry ? NameEntry : LinkageNameEntry;
}
/// Algorithm from llvm implementation.
@@ -382,6 +400,11 @@ void DWARF5AcceleratorTable::populateAbbrevsMap() {
if (SecondEntryRet)
Abbrev.addAttribute(SecondEntryRet->Encoding);
Abbrev.addAttribute({dwarf::DW_IDX_die_offset, dwarf::DW_FORM_ref4});
+ if (std::optional<uint64_t> Offset = Value->getParentDieOffset())
+ Abbrev.addAttribute({dwarf::DW_IDX_parent, dwarf::DW_FORM_ref4});
+ else
+ Abbrev.addAttribute(
+ {dwarf::DW_IDX_parent, dwarf::DW_FORM_flag_present});
FoldingSetNodeID ID;
Abbrev.Profile(ID);
void *InsertPos;
@@ -401,7 +424,11 @@ void DWARF5AcceleratorTable::populateAbbrevsMap() {
}
}
-void DWARF5AcceleratorTable::writeEntry(const BOLTDWARF5AccelTableData &Entry) {
+void DWARF5AcceleratorTable::writeEntry(BOLTDWARF5AccelTableData &Entry) {
+ const uint64_t EntryID = getEntryID(Entry);
+ if (EntryRelativeOffsets.find(EntryID) != EntryRelativeOffsets.end())
+ EntryRelativeOffsets[EntryID] = EntriesBuffer->size();
+
const std::optional<DWARF5AccelTable::UnitIndexAndEncoding> EntryRet =
getIndexForEntry(Entry);
// For forgeign type (FTU) units that need to refer to the FTU and to the CU.
@@ -456,6 +483,17 @@ void DWARF5AcceleratorTable::writeEntry(const BOLTDWARF5AccelTableData &Entry) {
llvm::endianness::little);
break;
}
+ case dwarf::DW_IDX_parent: {
+ assert(
+ (AttrEnc.Form == dwarf::DW_FORM_ref4 && Entry.getParentDieOffset()) ||
+ AttrEnc.Form == dwarf::DW_FORM_flag_present);
+ if (std::optional<uint64_t> ParentOffset = Entry.getParentDieOffset()) {
+ Entry.setPatchOffset(EntriesBuffer->size());
+ support::endian::write(*Entriestream, static_cast<uint32_t>(UINT32_MAX),
+ llvm::endianness::little);
+ }
+ break;
+ }
}
}
}
@@ -464,13 +502,34 @@ void DWARF5AcceleratorTable::writeEntries() {
for (auto &Bucket : getBuckets()) {
for (DWARF5AcceleratorTable::HashData *Hash : Bucket) {
Hash->EntryOffset = EntriesBuffer->size();
- for (const BOLTDWARF5AccelTableData *Value : Hash->Values) {
+ for (BOLTDWARF5AccelTableData *Value : Hash->Values) {
writeEntry(*Value);
}
support::endian::write(*Entriestream, static_cast<uint8_t>(0),
llvm::endianness::little);
}
}
+ // Patching parent offsets.
+ for (auto &Bucket : getBuckets()) {
+ for (DWARF5AcceleratorTable::HashData *Hash : Bucket) {
+ for (BOLTDWARF5AccelTableData *Entry : Hash->Values) {
+ std::optional<uint64_t> ParentOffset = Entry->getParentDieOffset();
+ if (!ParentOffset)
+ continue;
+ if (const auto Iter = EntryRelativeOffsets.find(*ParentOffset);
+ Iter != EntryRelativeOffsets.end()) {
+ const uint64_t PatchOffset = Entry->getPatchOffset();
+ uint32_t *Ptr = reinterpret_cast<uint32_t *>(
+ &EntriesBuffer.get()->data()[PatchOffset]);
+ *Ptr = Iter->second;
+ } else {
+ BC.errs() << "BOLT-WARNING: [internal-dwarf-warning]: Could not find "
+ "entry with offset "
+ << *ParentOffset << "\n";
+ }
+ }
+ }
+ }
}
void DWARF5AcceleratorTable::writeAugmentationString() {
diff --git a/bolt/test/X86/dwarf5-debug-names-generate-debug-names.test b/bolt/test/X86/dwarf5-debug-names-generate-debug-names.test
index b8789a6ad2300a..ae921d1e8b93bc 100644
--- a/bolt/test/X86/dwarf5-debug-names-generate-debug-names.test
+++ b/bolt/test/X86/dwarf5-debug-names-generate-debug-names.test
@@ -14,7 +14,7 @@
; BOLT: [[OFFSET2:0x[0-9a-f]*]]: Compile Unit
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0x103
+; BOLT-NEXT: Length: 0x109
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 2
@@ -22,7 +22,7 @@
; BOLT-NEXT: Foreign TU count: 0
; BOLT-NEXT: Bucket count: 8
; BOLT-NEXT: Name count: 8
-; BOLT-NEXT: Abbreviations table size: 0x19
+; BOLT-NEXT: Abbreviations table size: 0x1F
; BOLT-NEXT: Augmentation: 'BOLT'
; BOLT-NEXT: }
; BOLT-NEXT: Compilation Unit offsets [
@@ -34,16 +34,19 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 0 [
@@ -55,12 +58,14 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000033
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV1]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x0000007f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -73,6 +78,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x0000005e
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 3 {
@@ -83,6 +89,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000028
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -95,6 +102,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000028
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 5 {
@@ -105,6 +113,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000049
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 6 {
@@ -115,6 +124,7 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000028
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -127,6 +137,7 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000092
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -145,6 +156,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x0000005e
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
diff --git a/bolt/test/X86/dwarf5-debug-names.test b/bolt/test/X86/dwarf5-debug-names.test
index de0e88d7a36ae5..8e45ef3a948123 100644
--- a/bolt/test/X86/dwarf5-debug-names.test
+++ b/bolt/test/X86/dwarf5-debug-names.test
@@ -11,7 +11,7 @@
; BOLT: [[OFFSET2:0x[0-9a-f]*]]: Compile Unit
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0x1C2
+; BOLT-NEXT: Length: 0x1DA
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 2
@@ -19,7 +19,7 @@
; BOLT-NEXT: Foreign TU count: 0
; BOLT-NEXT: Bucket count: 14
; BOLT-NEXT: Name count: 15
-; BOLT-NEXT: Abbreviations table size: 0x29
+; BOLT-NEXT: Abbreviations table size: 0x3D
; BOLT-NEXT: Augmentation: 'BOLT'
; BOLT-NEXT: }
; BOLT-NEXT: Compilation Unit offsets [
@@ -31,27 +31,38 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_namespace
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV4:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV5:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
+; BOLT-NEXT: Abbreviation [[ABBREV6:0x[0-9a-f]*]] {
+; BOLT-NEXT: Tag: DW_TAG_structure_type
+; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
+; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_ref4
+; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 0 [
; BOLT-NEXT: Name 1 {
@@ -62,6 +73,7 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x0000002f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -74,6 +86,7 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x000000eb
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -81,17 +94,19 @@
; BOLT-NEXT: Name 3 {
; BOLT-NEXT: Hash: 0x8CFC710C
; BOLT-NEXT: String: {{.+}} "(anonymous namespace)"
-; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Entry @ [[ENTRY:0x[0-9a-f]*]] {
; BOLT-NEXT: Abbrev: [[ABBREV2]]
; BOLT-NEXT: Tag: DW_TAG_namespace
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000061
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV2]]
; BOLT-NEXT: Tag: DW_TAG_namespace
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000061
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 4 {
@@ -102,6 +117,7 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x0000005a
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -120,6 +136,7 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x000000c9
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 6 {
@@ -130,6 +147,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000033
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 7 {
@@ -140,12 +158,14 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x0000009f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV4]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x000000c5
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -158,6 +178,7 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x0000003f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 9 {
@@ -168,6 +189,7 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000024
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -180,6 +202,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000033
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -192,6 +215,7 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000024
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -207,12 +231,14 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x0000002f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV4]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x0000005d
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 13 {
@@ -223,12 +249,14 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000078
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV1]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000104
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 14 {
@@ -239,6 +267,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000073
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -250,10 +279,11 @@
; BOLT-NEXT: Hash: 0x59796A
; BOLT-NEXT: String: {{.+}} "t1"
; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Abbrev: [[ABBREV6]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000062
+; BOLT-NEXT: DW_IDX_parent: Entry @ [[ENTRY]]
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
diff --git a/bolt/test/X86/dwarf5-df-debug-names-generate-debug-names.test b/bolt/test/X86/dwarf5-df-debug-names-generate-debug-names.test
index 3be327b780e525..b0b6707a4ea5cb 100644
--- a/bolt/test/X86/dwarf5-df-debug-names-generate-debug-names.test
+++ b/bolt/test/X86/dwarf5-df-debug-names-generate-debug-names.test
@@ -17,7 +17,7 @@
; BOLT: [[OFFSET2:0x[0-9a-f]*]]: Compile Unit
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0x148
+; BOLT-NEXT: Length: 0x14E
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 2
@@ -25,7 +25,7 @@
; BOLT-NEXT: Foreign TU count: 0
; BOLT-NEXT: Bucket count: 11
; BOLT-NEXT: Name count: 11
-; BOLT-NEXT: Abbreviations table size: 0x19
+; BOLT-NEXT: Abbreviations table size: 0x1F
; BOLT-NEXT: Augmentation: 'BOLT'
; BOLT-NEXT: }
; BOLT-NEXT: Compilation Unit offsets [
@@ -37,16 +37,19 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 0 [
@@ -58,6 +61,7 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000029
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 2 {
@@ -68,6 +72,7 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x0000008c
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -80,6 +85,7 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000029
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 4 {
@@ -90,6 +96,7 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -102,6 +109,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000034
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -114,6 +122,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000034
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -129,6 +138,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000034
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -141,12 +151,14 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000025
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV2]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000025
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -159,6 +171,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000034
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 10 {
@@ -169,6 +182,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000057
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -187,6 +201,7 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
diff --git a/bolt/test/X86/dwarf5-df-debug-names.test b/bolt/test/X86/dwarf5-df-debug-names.test
index 0352d0ff720822..d53f2bd13620fc 100644
--- a/bolt/test/X86/dwarf5-df-debug-names.test
+++ b/bolt/test/X86/dwarf5-df-debug-names.test
@@ -14,136 +14,148 @@
; BOLT: [[OFFSET:0x[0-9a-f]*]]: Compile Unit
; BOLT: [[OFFSET1:0x[0-9a-f]*]]: Compile Unit
-: BOLT: Name Index @ 0x0 {
-: BOLT-NEXT: Header {
-: BOLT-NEXT: Length: 0xF4
-: BOLT-NEXT: Format: DWARF32
-: BOLT-NEXT: Version: 5
-: BOLT-NEXT: CU count: 2
-: BOLT-NEXT: Local TU count: 0
-: BOLT-NEXT: Foreign TU count: 0
-: BOLT-NEXT: Bucket count: 7
-: BOLT-NEXT: Name count: 7
-: BOLT-NEXT: Abbreviations table size: 0x21
-: BOLT-NEXT: Augmentation: 'BOLT'
-: BOLT-NEXT: }
-: BOLT-NEXT: Compilation Unit offsets [
-: BOLT-NEXT: CU[0]: [[OFFSET]]
-: BOLT-NEXT: CU[1]: [[OFFSET1]]
-: BOLT-NEXT: ]
-: BOLT-NEXT: Abbreviations [
-: BOLT-NEXT: Abbreviation [[ABBREV1:0x[0-9a-f]*]] {
-: BOLT-NEXT: Tag: DW_TAG_structure_type
-: BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
-: BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
-: BOLT-NEXT: }
-: BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
-: BOLT-NEXT: Tag: DW_TAG_base_type
-: BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
-: BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
-: BOLT-NEXT: }
-: BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
-: BOLT-NEXT: Tag: DW_TAG_variable
-: BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
-: BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
-: BOLT-NEXT: }
-: BOLT-NEXT: Abbreviation [[ABBREV4:0x[0-9a-f]*]] {
-: BOLT-NEXT: Tag: DW_TAG_subprogram
-: BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
-: BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
-: BOLT-NEXT: }
-: BOLT-NEXT: ]
-: BOLT-NEXT: Bucket 0 [
-: BOLT-NEXT: EMPTY
-: BOLT-NEXT: ]
-: BOLT-NEXT: Bucket 1 [
-: BOLT-NEXT: Name 1 {
-: BOLT-NEXT: Hash: 0x7C96E4DB
-: BOLT-NEXT: String: {{.+}} "Foo2"
-: BOLT-NEXT: Entry @ {{.+}} {
-: BOLT-NEXT: Abbrev: [[ABBREV1]]
-: BOLT-NEXT: Tag: DW_TAG_structure_type
-: BOLT-NEXT: DW_IDX_compile_unit: 0x00
-: BOLT-NEXT: DW_IDX_die_offset: 0x00000068
-: BOLT-NEXT: }
-: BOLT-NEXT: }
-: BOLT-NEXT: ]
-: BOLT-NEXT: Bucket 2 [
-: BOLT-NEXT: Name 2 {
-: BOLT-NEXT: Hash: 0xBA564846
-: BOLT-NEXT: String: {{.+}} "Foo2Int"
-: BOLT-NEXT: Entry @ {{.+}} {
-: BOLT-NEXT: Abbrev: [[ABBREV1]]
-: BOLT-NEXT: Tag: DW_TAG_structure_type
-: BOLT-NEXT: DW_IDX_compile_unit: 0x01
-: BOLT-NEXT: DW_IDX_die_offset: 0x00000025
-: BOLT-NEXT: }
-: BOLT-NEXT: }
-: BOLT-NEXT: ]
-: BOLT-NEXT: Bucket 3 [
-: BOLT-NEXT: Name 3 {
-: BOLT-NEXT: Hash: 0xB888030
-: BOLT-NEXT: String: {{.+}} "int"
-: BOLT-NEXT: Entry @ {{.+}} {
-: BOLT-NEXT: Abbrev: [[ABBREV2]]
-: BOLT-NEXT: Tag: DW_TAG_base_type
-: BOLT-NEXT: DW_IDX_compile_unit: 0x01
-: BOLT-NEXT: DW_IDX_die_offset: 0x00000043
-: BOLT-NEXT: }
-: BOLT-NEXT: Entry @ {{.+}} {
-: BOLT-NEXT: Abbrev: [[ABBREV2]]
-: BOLT-NEXT: Tag: DW_TAG_base_type
-: BOLT-NEXT: DW_IDX_compile_unit: 0x00
-: BOLT-NEXT: DW_IDX_die_offset: 0x00000056
-: BOLT-NEXT: }
-: BOLT-NEXT: }
-: BOLT-NEXT: Name 4 {
-: BOLT-NEXT: Hash: 0xF73809C
-: BOLT-NEXT: String: {{.+}} "Foo2a"
-: BOLT-NEXT: Entry @ {{.+}} {
-: BOLT-NEXT: Abbrev: [[ABBREV1]]
-: BOLT-NEXT: Tag: DW_TAG_structure_type
-: BOLT-NEXT: DW_IDX_compile_unit: 0x00
-: BOLT-NEXT: DW_IDX_die_offset: 0x00000078
-: BOLT-NEXT: }
-: BOLT-NEXT: }
-: BOLT-NEXT: Name 5 {
-: BOLT-NEXT: Hash: 0x7C96CB76
-: BOLT-NEXT: String: {{.+}} "fint"
-: BOLT-NEXT: Entry @ {{.+}} {
-: BOLT-NEXT: Abbrev: [[ABBREV3]]
-: BOLT-NEXT: Tag: DW_TAG_variable
-: BOLT-NEXT: DW_IDX_compile_unit: 0x01
-: BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
-: BOLT-NEXT: }
-: BOLT-NEXT: }
-: BOLT-NEXT: Name 6 {
-: BOLT-NEXT: Hash: 0x7C9A7F6A
-: BOLT-NEXT: String: {{.+}} "main"
-: BOLT-NEXT: Entry @ {{.+}} {
-: BOLT-NEXT: Abbrev: [[ABBREV4]]
-: BOLT-NEXT: Tag: DW_TAG_subprogram
-: BOLT-NEXT: DW_IDX_compile_unit: 0x00
-: BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
-: BOLT-NEXT: }
-: BOLT-NEXT: }
-: BOLT-NEXT: ]
-: BOLT-NEXT: Bucket 4 [
-: BOLT-NEXT: EMPTY
-: BOLT-NEXT: ]
-: BOLT-NEXT: Bucket 5 [
-: BOLT-NEXT: Name 7 {
-: BOLT-NEXT: Hash: 0x7C952063
-: BOLT-NEXT: String: {{.+}} "char"
-: BOLT-NEXT: Entry @ {{.+}} {
-: BOLT-NEXT: Abbrev: [[ABBREV2]]
-: BOLT-NEXT: Tag: DW_TAG_base_type
-: BOLT-NEXT: DW_IDX_compile_unit: 0x00
-: BOLT-NEXT: DW_IDX_die_offset: 0x00000064
-: BOLT-NEXT: }
-: BOLT-NEXT: }
-: BOLT-NEXT: ]
-: BOLT-NEXT: Bucket 6 [
-: BOLT-NEXT: EMPTY
-: BOLT-NEXT: ]
-: BOLT-NEXT: }
+; BOLT: Name Index @ 0x0 {
+; BOLT-NEXT: Header {
+; BOLT-NEXT: Length: 0xFC
+; BOLT-NEXT: Format: DWARF32
+; BOLT-NEXT: Version: 5
+; BOLT-NEXT: CU count: 2
+; BOLT-NEXT: Local TU count: 0
+; BOLT-NEXT: Foreign TU count: 0
+; BOLT-NEXT: Bucket count: 7
+; BOLT-NEXT: Name count: 7
+; BOLT-NEXT: Abbreviations table size: 0x29
+; BOLT-NEXT: Augmentation: 'BOLT'
+; BOLT-NEXT: }
+; BOLT-NEXT: Compilation Unit offsets [
+; BOLT-NEXT: CU[0]: [[OFFSET]]
+; BOLT-NEXT: CU[1]: [[OFFSET1]]
+; BOLT-NEXT: ]
+; BOLT-NEXT: Abbreviations [
+; BOLT-NEXT: Abbreviation [[ABBREV1:0x[0-9a-f]*]] {
+; BOLT-NEXT: Tag: DW_TAG_structure_type
+; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
+; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
+; BOLT-NEXT: }
+; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
+; BOLT-NEXT: Tag: DW_TAG_base_type
+; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
+; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
+; BOLT-NEXT: }
+; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
+; BOLT-NEXT: Tag: DW_TAG_variable
+; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
+; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
+; BOLT-NEXT: }
+; BOLT-NEXT: Abbreviation [[ABBREV4:0x[0-9a-f]*]] {
+; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
+; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
+; BOLT-NEXT: }
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 0 [
+; BOLT-NEXT: EMPTY
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 1 [
+; BOLT-NEXT: Name 1 {
+; BOLT-NEXT: Hash: 0x7C96E4DB
+; BOLT-NEXT: String: {{.+}} "Foo2"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_structure_type
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000068
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: }
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 2 [
+; BOLT-NEXT: Name 2 {
+; BOLT-NEXT: Hash: 0xBA564846
+; BOLT-NEXT: String: {{.+}} "Foo2Int"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_structure_type
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000025
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: }
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 3 [
+; BOLT-NEXT: Name 3 {
+; BOLT-NEXT: Hash: 0xB888030
+; BOLT-NEXT: String: {{.+}} "int"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV2]]
+; BOLT-NEXT: Tag: DW_TAG_base_type
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000043
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV2]]
+; BOLT-NEXT: Tag: DW_TAG_base_type
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000056
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: }
+; BOLT-NEXT: Name 4 {
+; BOLT-NEXT: Hash: 0xF73809C
+; BOLT-NEXT: String: {{.+}} "Foo2a"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_structure_type
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000078
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: }
+; BOLT-NEXT: Name 5 {
+; BOLT-NEXT: Hash: 0x7C96CB76
+; BOLT-NEXT: String: {{.+}} "fint"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV3]]
+; BOLT-NEXT: Tag: DW_TAG_variable
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: }
+; BOLT-NEXT: Name 6 {
+; BOLT-NEXT: Hash: 0x7C9A7F6A
+; BOLT-NEXT: String: {{.+}} "main"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV4]]
+; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: }
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 4 [
+; BOLT-NEXT: EMPTY
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 5 [
+; BOLT-NEXT: Name 7 {
+; BOLT-NEXT: Hash: 0x7C952063
+; BOLT-NEXT: String: {{.+}} "char"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV2]]
+; BOLT-NEXT: Tag: DW_TAG_base_type
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000064
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: }
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 6 [
+; BOLT-NEXT: EMPTY
+; BOLT-NEXT: ]
+; BOLT-NEXT: }
diff --git a/bolt/test/X86/dwarf5-df-main-debug-names-ftu-ltu-mix.test b/bolt/test/X86/dwarf5-df-main-debug-names-ftu-ltu-mix.test
index 8a8a4b118b8c03..3a0260fd0bfb91 100644
--- a/bolt/test/X86/dwarf5-df-main-debug-names-ftu-ltu-mix.test
+++ b/bolt/test/X86/dwarf5-df-main-debug-names-ftu-ltu-mix.test
@@ -31,6 +31,7 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: 0x02
; BOLT-NEXT: DW_IDX_die_offset: 0x0000001e
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT: Name 6 {
@@ -42,6 +43,7 @@
; BOLT-NEXT: DW_IDX_type_unit: 0x02
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000021
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT: Name 7 {
@@ -52,5 +54,6 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_type_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000023
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
diff --git a/bolt/test/X86/dwarf5-df-one-cu-debug-names.test b/bolt/test/X86/dwarf5-df-one-cu-debug-names.test
index 246ce7efb3bad6..bf1cad81040bd9 100644
--- a/bolt/test/X86/dwarf5-df-one-cu-debug-names.test
+++ b/bolt/test/X86/dwarf5-df-one-cu-debug-names.test
@@ -13,7 +13,7 @@
; BOLT: [[OFFSET:0x[0-9a-f]*]]: Compile Unit
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0xA9
+; BOLT-NEXT: Length: 0xAF
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 1
@@ -21,7 +21,7 @@
; BOLT-NEXT: Foreign TU count: 0
; BOLT-NEXT: Bucket count: 5
; BOLT-NEXT: Name count: 5
-; BOLT-NEXT: Abbreviations table size: 0x13
+; BOLT-NEXT: Abbreviations table size: 0x19
; BOLT-NEXT: Augmentation: 'BOLT'
; BOLT-NEXT: }
; BOLT-NEXT: Compilation Unit offsets [
@@ -31,14 +31,17 @@
; BOLT-NEXT: Abbreviation [[ABBREV1:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 0 [
@@ -52,6 +55,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV1]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: 0x00000068
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 2 {
@@ -61,6 +65,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV2]]
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -75,6 +80,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV3]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: 0x00000056
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -86,6 +92,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV1]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: 0x00000078
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 5 {
@@ -95,6 +102,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV3]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: 0x00000064
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
diff --git a/bolt/test/X86/dwarf5-df-types-debug-names.test b/bolt/test/X86/dwarf5-df-types-debug-names.test
index fdb962b80c7512..f5a2c9c10353e9 100644
--- a/bolt/test/X86/dwarf5-df-types-debug-names.test
+++ b/bolt/test/X86/dwarf5-df-types-debug-names.test
@@ -25,7 +25,7 @@
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0x174
+; BOLT-NEXT: Length: 0x17E
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 2
@@ -33,7 +33,7 @@
; BOLT-NEXT: Foreign TU count: 4
; BOLT-NEXT: Bucket count: 9
; BOLT-NEXT: Name count: 9
-; BOLT-NEXT: Abbreviations table size: 0x2D
+; BOLT-NEXT: Abbreviations table size: 0x37
; BOLT-NEXT: Augmentation: 'BOLT'
; BOLT-NEXT: }
; BOLT-NEXT: Compilation Unit offsets [
@@ -52,27 +52,32 @@
; BOLT-NEXT: DW_IDX_type_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV1:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV4:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_type_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 0 [
@@ -88,6 +93,7 @@
; BOLT-NEXT: DW_IDX_type_unit: 0x00
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000021
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 2 {
@@ -98,6 +104,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000029
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 3 {
@@ -108,6 +115,7 @@
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -120,6 +128,7 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000025
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: 0x5
@@ -127,12 +136,14 @@
; BOLT-NEXT: DW_IDX_type_unit: 0x02
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x0000003f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV3]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000056
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -145,6 +156,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000029
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 6 {
@@ -156,6 +168,7 @@
; BOLT-NEXT: DW_IDX_type_unit: 0x01
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000021
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV]]
@@ -163,6 +176,7 @@
; BOLT-NEXT: DW_IDX_type_unit: 0x03
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000021
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 7 {
@@ -174,6 +188,7 @@
; BOLT-NEXT: DW_IDX_type_unit: 0x02
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000021
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -195,6 +210,7 @@
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -208,6 +224,7 @@
; BOLT-NEXT: DW_IDX_type_unit: 0x00
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000036
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: 0x5
@@ -215,6 +232,7 @@
; BOLT-NEXT: DW_IDX_type_unit: 0x01
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000048
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: 0x5
@@ -222,12 +240,14 @@
; BOLT-NEXT: DW_IDX_type_unit: 0x03
; BOLT-NEXT: DW_IDX_compile_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000048
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV3]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000064
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
diff --git a/bolt/test/X86/dwarf5-df-types-one-cu-debug-names.test b/bolt/test/X86/dwarf5-df-types-one-cu-debug-names.test
index 1c77583f50883f..2a489d86f5d7de 100644
--- a/bolt/test/X86/dwarf5-df-types-one-cu-debug-names.test
+++ b/bolt/test/X86/dwarf5-df-types-one-cu-debug-names.test
@@ -17,7 +17,7 @@
; BOLT: [[OFFSET:0x[0-9a-f]*]]: Compile Unit
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0xD1
+; BOLT-NEXT: Length: 0xD9
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 1
@@ -25,7 +25,7 @@
; BOLT-NEXT: Foreign TU count: 2
; BOLT-NEXT: Bucket count: 5
; BOLT-NEXT: Name count: 5
-; BOLT-NEXT: Abbreviations table size: 0x1D
+; BOLT-NEXT: Abbreviations table size: 0x25
; BOLT-NEXT: Augmentation: 'BOLT'
; BOLT-NEXT: }
; BOLT-NEXT: Compilation Unit offsets [
@@ -40,19 +40,23 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_type_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV4:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_type_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 0 [
@@ -67,6 +71,7 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_type_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000021
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 2 {
@@ -76,6 +81,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV2]]
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_die_offset: 0x0000001a
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -90,6 +96,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV3]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: 0x00000056
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -102,6 +109,7 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_type_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000021
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 5 {
@@ -112,17 +120,20 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_type_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000036
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV4]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_type_unit: 0x01
; BOLT-NEXT: DW_IDX_die_offset: 0x00000048
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
; BOLT-NEXT: Abbrev: [[ABBREV3]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: 0x00000064
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
diff --git a/bolt/test/X86/dwarf5-one-cu-debug-names.test b/bolt/test/X86/dwarf5-one-cu-debug-names.test
index e7754521d61cbe..2145d8160a42dc 100644
--- a/bolt/test/X86/dwarf5-one-cu-debug-names.test
+++ b/bolt/test/X86/dwarf5-one-cu-debug-names.test
@@ -9,7 +9,7 @@
; BOLT: [[OFFSET1:0x[0-9a-f]*]]: Compile Unit
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0x13E
+; BOLT-NEXT: Length: 0x154
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 1
@@ -17,7 +17,7 @@
; BOLT-NEXT: Foreign TU count: 0
; BOLT-NEXT: Bucket count: 11
; BOLT-NEXT: Name count: 11
-; BOLT-NEXT: Abbreviations table size: 0x1F
+; BOLT-NEXT: Abbreviations table size: 0x31
; BOLT-NEXT: Augmentation: 'BOLT'
; BOLT-NEXT: }
; BOLT-NEXT: Compilation Unit offsets [
@@ -27,22 +27,32 @@
; BOLT-NEXT: Abbreviation [[ABBREV1:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
+; BOLT-NEXT: Tag: DW_TAG_structure_type
+; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_ref4
+; BOLT-NEXT: }
+; BOLT-NEXT: Abbreviation [[ABBREV4:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
-; BOLT-NEXT: Abbreviation [[ABBREV4:0x[0-9a-f]*]] {
+; BOLT-NEXT: Abbreviation [[ABBREV5:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
-; BOLT-NEXT: Abbreviation [[ABBREV5:0x[0-9a-f]*]] {
+; BOLT-NEXT: Abbreviation [[ABBREV6:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_namespace
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 0 [
@@ -53,6 +63,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV1]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: 0x00000104
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 2 {
@@ -62,6 +73,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV2]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: 0x000000c5
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -73,6 +85,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV1]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: 0x000000c9
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 4 {
@@ -82,6 +95,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV1]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: 0x0000003f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -93,6 +107,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV1]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: 0x000000eb
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -107,18 +122,20 @@
; BOLT-NEXT: Hash: 0x59796A
; BOLT-NEXT: String: {{.+}} "t1"
; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Abbrev: [[ABBREV3]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: 0x00000062
+; BOLT-NEXT: DW_IDX_parent: Entry @ 0x14d
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 7 {
; BOLT-NEXT: Hash: 0x5979AC
; BOLT-NEXT: String: {{.+}} "v1"
; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV3]]
+; BOLT-NEXT: Abbrev: [[ABBREV4]]
; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_die_offset: 0x00000024
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -130,6 +147,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV2]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: 0x0000005d
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -141,15 +159,17 @@
; BOLT-NEXT: Abbrev: [[ABBREV1]]
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_die_offset: 0x0000002f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 10 {
; BOLT-NEXT: Hash: 0x7C9A7F6A
; BOLT-NEXT: String: {{.+}} "main"
; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV4]]
+; BOLT-NEXT: Abbrev: [[ABBREV5]]
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_die_offset: 0x00000073
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -157,15 +177,17 @@
; BOLT-NEXT: Name 11 {
; BOLT-NEXT: Hash: 0x8CFC710C
; BOLT-NEXT: String: {{.+}} "(anonymous namespace)"
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV5]]
+; BOLT-NEXT: Entry @ 0x14d {
+; BOLT-NEXT: Abbrev: [[ABBREV6]]
; BOLT-NEXT: Tag: DW_TAG_namespace
; BOLT-NEXT: DW_IDX_die_offset: 0x00000061
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV5]]
+; BOLT-NEXT: Abbrev: [[ABBREV6]]
; BOLT-NEXT: Tag: DW_TAG_namespace
; BOLT-NEXT: DW_IDX_die_offset: 0x00000061
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
diff --git a/bolt/test/X86/dwarf5-types-debug-names.test b/bolt/test/X86/dwarf5-types-debug-names.test
index 6a26477d4cdb55..94624298e289de 100644
--- a/bolt/test/X86/dwarf5-types-debug-names.test
+++ b/bolt/test/X86/dwarf5-types-debug-names.test
@@ -14,7 +14,7 @@
; BOLT: Name Index @ 0x0 {
; BOLT: Header {
-; BOLT: Length: 0xE1
+; BOLT: Length: 0xE9
; BOLT: Format: DWARF32
; BOLT: Version: 5
; BOLT: CU count: 2
@@ -22,7 +22,7 @@
; BOLT: Foreign TU count: 0
; BOLT: Bucket count: 6
; BOLT: Name count: 6
-; BOLT: Abbreviations table size: 0x21
+; BOLT: Abbreviations table size: 0x29
; BOLT: Augmentation: 'BOLT'
; BOLT: }
; BOLT: Compilation Unit offsets [
@@ -37,21 +37,25 @@
; BOLT: Tag: DW_TAG_structure_type
; BOLT: DW_IDX_type_unit: DW_FORM_data1
; BOLT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT: DW_IDX_parent: DW_FORM_flag_present
; BOLT: }
; BOLT: Abbreviation [[ABBREV1:0x[0-9a-f]*]] {
; BOLT: Tag: DW_TAG_subprogram
; BOLT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT: DW_IDX_parent: DW_FORM_flag_present
; BOLT: }
; BOLT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
; BOLT: Tag: DW_TAG_base_type
; BOLT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT: DW_IDX_parent: DW_FORM_flag_present
; BOLT: }
; BOLT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
; BOLT: Tag: DW_TAG_base_type
; BOLT: DW_IDX_type_unit: DW_FORM_data1
; BOLT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT: DW_IDX_parent: DW_FORM_flag_present
; BOLT: }
; BOLT: ]
; BOLT: Bucket 0 [
@@ -63,6 +67,7 @@
; BOLT: Tag: DW_TAG_structure_type
; BOLT: DW_IDX_type_unit: 0x00
; BOLT: DW_IDX_die_offset: 0x00000023
+; BOLT: DW_IDX_parent: <parent not indexed>
; BOLT: }
; BOLT: }
; BOLT: ]
@@ -75,6 +80,7 @@
; BOLT: Tag: DW_TAG_subprogram
; BOLT: DW_IDX_compile_unit: 0x01
; BOLT: DW_IDX_die_offset: 0x00000024
+; BOLT: DW_IDX_parent: <parent not indexed>
; BOLT: }
; BOLT: }
; BOLT: ]
@@ -87,6 +93,7 @@
; BOLT: Tag: DW_TAG_base_type
; BOLT: DW_IDX_compile_unit: 0x01
; BOLT: DW_IDX_die_offset: 0x00000040
+; BOLT: DW_IDX_parent: <parent not indexed>
; BOLT: }
; BOLT: }
; BOLT: ]
@@ -99,6 +106,7 @@
; BOLT: Tag: DW_TAG_subprogram
; BOLT: DW_IDX_compile_unit: 0x01
; BOLT: DW_IDX_die_offset: 0x00000024
+; BOLT: DW_IDX_parent: <parent not indexed>
; BOLT: }
; BOLT: }
; BOLT: ]
@@ -111,6 +119,7 @@
; BOLT: Tag: DW_TAG_subprogram
; BOLT: DW_IDX_compile_unit: 0x00
; BOLT: DW_IDX_die_offset: 0x00000024
+; BOLT: DW_IDX_parent: <parent not indexed>
; BOLT: }
; BOLT: }
; BOLT: ]
@@ -123,6 +132,7 @@
; BOLT: Tag: DW_TAG_base_type
; BOLT: DW_IDX_type_unit: 0x00
; BOLT: DW_IDX_die_offset: 0x00000038
+; BOLT: DW_IDX_parent: <parent not indexed>
; BOLT: }
; BOLT: }
; BOLT: ]
diff --git a/bolt/test/X86/dwarf5-types-one-cu-debug-names.test b/bolt/test/X86/dwarf5-types-one-cu-debug-names.test
index 00a1319a557869..02c0ef81351f1f 100644
--- a/bolt/test/X86/dwarf5-types-one-cu-debug-names.test
+++ b/bolt/test/X86/dwarf5-types-one-cu-debug-names.test
@@ -11,7 +11,7 @@
; BOLT:Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0xA3
+; BOLT-NEXT: Length: 0xAB
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 1
@@ -19,7 +19,7 @@
; BOLT-NEXT: Foreign TU count: 0
; BOLT-NEXT: Bucket count: 4
; BOLT-NEXT: Name count: 4
-; BOLT-NEXT: Abbreviations table size: 0x1D
+; BOLT-NEXT: Abbreviations table size: 0x25
; BOLT-NEXT: Augmentation: 'BOLT'
; BOLT-NEXT: }
; BOLT-NEXT: Compilation Unit offsets [
@@ -32,20 +32,24 @@
; BOLT-NEXT: Abbreviation [[ABBREV:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV1:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_type_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_type_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 0 [
@@ -56,6 +60,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV]]
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_die_offset: 0x0000003f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 2 {
@@ -66,6 +71,7 @@
; BOLT-NEXT: Tag: DW_TAG_structure_type
; BOLT-NEXT: DW_IDX_type_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000023
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -80,6 +86,7 @@
; BOLT-NEXT: Abbrev: [[ABBREV2]]
; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_die_offset: 0x00000024
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
@@ -92,6 +99,7 @@
; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_type_unit: 0x00
; BOLT-NEXT: DW_IDX_die_offset: 0x00000038
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
>From 7b3b001dbd175631f824d1a59693ce8eb26a43c8 Mon Sep 17 00:00:00 2001
From: Alexander Yermolovich <ayermolo at meta.com>
Date: Fri, 15 Mar 2024 16:41:45 -0700
Subject: [PATCH 2/2] [BOLT][DWARF] Add support to debug_names for
DW_AT_abstract_origin/DW_AT_specification
Summary:
According to the DWARF spec a DIE that has DW_AT_specification or
DW_AT_abstract_origin can be part of .debug_name if a DIE those
attribute points to has DW_AT_name or DW_AT_linkage_name.
Test Plan: ninja check-bolt
Reviewers: #llvm-bolt
Differential Revision: https://phabricator.intern.facebook.com/D54975714
Tasks: T181824698
Tags: llvm-bolt
---
bolt/lib/Core/DebugNames.cpp | 46 +-
.../dwarf5-debug-names-dw-at-specification.s | 694 ++++++++++++++++++
...arf5-debug-names-generate-debug-names.test | 273 ++++---
bolt/test/X86/dwarf5-debug-names.test | 9 +-
bolt/test/X86/dwarf5-one-cu-debug-names.test | 8 +-
5 files changed, 894 insertions(+), 136 deletions(-)
create mode 100644 bolt/test/X86/dwarf5-debug-names-dw-at-specification.s
diff --git a/bolt/lib/Core/DebugNames.cpp b/bolt/lib/Core/DebugNames.cpp
index 4bcedbea8d2252..d94db481a59183 100644
--- a/bolt/lib/Core/DebugNames.cpp
+++ b/bolt/lib/Core/DebugNames.cpp
@@ -234,8 +234,7 @@ DWARF5AcceleratorTable::addAccelTableEntry(
addUnit(Unit, DWOID);
}
- auto addEntry =
- [&](DIEValue ValName) -> std::optional<BOLTDWARF5AccelTableData *> {
+ auto getName = [&](DIEValue ValName) -> std::optional<std::string> {
if ((!ValName || ValName.getForm() == dwarf::DW_FORM_string) &&
NameToUse.empty())
return std::nullopt;
@@ -268,7 +267,16 @@ DWARF5AcceleratorTable::addAccelTableEntry(
// This the same hash function used in DWARF5AccelTableData.
It.HashValue = caseFoldingDjbHash(Name);
}
+ return Name;
+ };
+
+ auto addEntry =
+ [&](DIEValue ValName) -> std::optional<BOLTDWARF5AccelTableData *> {
+ std::optional<std::string> Name = getName(ValName);
+ if (!Name)
+ return std::nullopt;
+ auto &It = Entries[*Name];
bool IsTU = false;
uint32_t DieTag = 0;
uint32_t UnitID = getUnitID(Unit, IsTU, DieTag);
@@ -278,7 +286,7 @@ DWARF5AcceleratorTable::addAccelTableEntry(
if (Iter == CUOffsetsToPatch.end())
BC.errs() << "BOLT-WARNING: [internal-dwarf-warning]: Could not find "
"DWO ID in CU offsets for second Unit Index "
- << Name << ". For DIE at offset: "
+ << *Name << ". For DIE at offset: "
<< Twine::utohexstr(CurrentUnitOffset + Die.getOffset())
<< ".\n";
SecondIndex = Iter->second;
@@ -295,11 +303,33 @@ DWARF5AcceleratorTable::addAccelTableEntry(
return It.Values.back();
};
- std::optional<BOLTDWARF5AccelTableData *> NameEntry =
- addEntry(Die.findAttribute(dwarf::Attribute::DW_AT_name));
- std::optional<BOLTDWARF5AccelTableData *> LinkageNameEntry =
- addEntry(Die.findAttribute(dwarf::Attribute::DW_AT_linkage_name));
- return NameEntry ? NameEntry : LinkageNameEntry;
+ // Minor optimization not to add entry twice for DW_TAG_namespace if it has no
+ // DW_AT_name.
+ if (!(Die.getTag() == dwarf::DW_TAG_namespace &&
+ !Die.findAttribute(dwarf::Attribute::DW_AT_name)))
+ addEntry(Die.findAttribute(dwarf::Attribute::DW_AT_linkage_name));
+ // For the purposes of determining whether a debugging information entry has a
+ // particular attribute (such as DW_AT_name), if debugging information entry A
+ // has a DW_AT_specification or DW_AT_abstract_origin attribute pointing to
+ // another debugging information entry B, any attributes of B are considered
+ // to be part of A.
+ if (DIEValue AbstrOrigin =
+ Die.findAttribute(dwarf::Attribute::DW_AT_abstract_origin)) {
+ const DIEEntry &DIEENtry = AbstrOrigin.getDIEEntry();
+ DIE &EntryDie = DIEENtry.getEntry();
+ addEntry(EntryDie.findAttribute(dwarf::Attribute::DW_AT_linkage_name));
+ return addEntry(EntryDie.findAttribute(dwarf::Attribute::DW_AT_name));
+ }
+ if (DIEValue AbstrOrigin =
+ Die.findAttribute(dwarf::Attribute::DW_AT_specification)) {
+ const DIEEntry &DIEENtry = AbstrOrigin.getDIEEntry();
+ DIE &EntryDie = DIEENtry.getEntry();
+ addEntry(EntryDie.findAttribute(dwarf::Attribute::DW_AT_linkage_name));
+ return addEntry(EntryDie.findAttribute(dwarf::Attribute::DW_AT_name));
+ }
+
+ return addEntry(Die.findAttribute(dwarf::Attribute::DW_AT_name));
+ ;
}
/// Algorithm from llvm implementation.
diff --git a/bolt/test/X86/dwarf5-debug-names-dw-at-specification.s b/bolt/test/X86/dwarf5-debug-names-dw-at-specification.s
new file mode 100644
index 00000000000000..ec5b425625ce96
--- /dev/null
+++ b/bolt/test/X86/dwarf5-debug-names-dw-at-specification.s
@@ -0,0 +1,694 @@
+# RUN: llvm-mc -dwarf-version=5 -filetype=obj -triple x86_64-unknown-linux %s -o %tmain.o
+# RUN: %clang %cflags -gdwarf-5 %tmain.o -o %tmain.exe
+# RUN: llvm-bolt %tmain.exe -o %tmain.exe.bolt --update-debug-sections
+# RUN: llvm-dwarfdump --debug-info -r 0 --debug-names %tmain.exe.bolt > %tlog.txt
+# RUN: cat %tlog.txt | FileCheck -check-prefix=BOLT %s
+
+## Tests that BOLT correctly generates entries in .debug_names with DW_AT_specification.
+
+# BOLT: [[OFFSET1:0x[0-9a-f]*]]: Compile Unit
+# BOLT: Name Index @ 0x0
+# BOLT-NEXT: Header {
+# BOLT-NEXT: Length: 0x10F
+# BOLT-NEXT: Format: DWARF32
+# BOLT-NEXT: Version: 5
+# BOLT-NEXT: CU count: 1
+# BOLT-NEXT: Local TU count: 0
+# BOLT-NEXT: Foreign TU count: 0
+# BOLT-NEXT: Bucket count: 9
+# BOLT-NEXT: Name count: 9
+# BOLT-NEXT: Abbreviations table size: 0x21
+# BOLT-NEXT: Augmentation: 'BOLT'
+# BOLT-NEXT: }
+# BOLT-NEXT: Compilation Unit offsets [
+# BOLT-NEXT: CU[0]: [[OFFSET1]]
+# BOLT-NEXT: ]
+# BOLT-NEXT: Abbreviations [
+# BOLT-NEXT: Abbreviation [[ABBREV1:0x[0-9a-f]*]] {
+# BOLT-NEXT: Tag: DW_TAG_variable
+# BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+# BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
+# BOLT-NEXT: }
+# BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
+# BOLT-NEXT: Tag: DW_TAG_structure_type
+# BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+# BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
+# BOLT-NEXT: }
+# BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
+# BOLT-NEXT: Tag: DW_TAG_base_type
+# BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+# BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
+# BOLT-NEXT: }
+# BOLT-NEXT: Abbreviation [[ABBREV4:0x[0-9a-f]*]] {
+# BOLT-NEXT: Tag: DW_TAG_subprogram
+# BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+# BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
+# BOLT-NEXT: }
+# BOLT-NEXT: ]
+# BOLT-NEXT: Bucket 0 [
+# BOLT-NEXT: Name 1 {
+# BOLT-NEXT: Hash: 0x5D3CA9E0
+# BOLT-NEXT: String: {{.+}} "_ZN1A15fully_specifiedE"
+# BOLT-NEXT: Entry @ {{.+}} {
+# BOLT-NEXT: Abbrev: [[ABBREV1]]
+# BOLT-NEXT: Tag: DW_TAG_variable
+# BOLT-NEXT: DW_IDX_die_offset: 0x00000024
+# BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+# BOLT-NEXT: }
+# BOLT-NEXT: }
+# BOLT-NEXT: Name 2 {
+# BOLT-NEXT: Hash: 0x7C9DFC37
+# BOLT-NEXT: String: {{.+}} "smem"
+# BOLT-NEXT: Entry @ {{.+}} {
+# BOLT-NEXT: Abbrev: [[ABBREV1]]
+# BOLT-NEXT: Tag: DW_TAG_variable
+# BOLT-NEXT: DW_IDX_die_offset: 0x00000057
+# BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+# BOLT-NEXT: }
+# BOLT-NEXT: }
+# BOLT-NEXT: ]
+# BOLT-NEXT: Bucket 1 [
+# BOLT-NEXT: Name 3 {
+# BOLT-NEXT: Hash: 0x2B606
+# BOLT-NEXT: String: {{.+}} "A"
+# BOLT-NEXT: Entry @ {{.+}} {
+# BOLT-NEXT: Abbrev: [[ABBREV2]]
+# BOLT-NEXT: Tag: DW_TAG_structure_type
+# BOLT-NEXT: DW_IDX_die_offset: 0x0000002d
+# BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+# BOLT-NEXT: }
+# BOLT-NEXT: }
+# BOLT-NEXT: ]
+# BOLT-NEXT: Bucket 2 [
+# BOLT-NEXT: Name 4 {
+# BOLT-NEXT: Hash: 0xB888030
+# BOLT-NEXT: String: {{.+}} "int"
+# BOLT-NEXT: Entry @ {{.+}} {
+# BOLT-NEXT: Abbrev: [[ABBREV3]]
+# BOLT-NEXT: Tag: DW_TAG_base_type
+# BOLT-NEXT: DW_IDX_die_offset: 0x00000044
+# BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+# BOLT-NEXT: }
+# BOLT-NEXT: }
+# BOLT-NEXT: ]
+# BOLT-NEXT: Bucket 3 [
+# BOLT-NEXT: EMPTY
+# BOLT-NEXT: ]
+# BOLT-NEXT: Bucket 4 [
+# BOLT-NEXT: EMPTY
+# BOLT-NEXT: ]
+# BOLT-NEXT: Bucket 5 [
+# BOLT-NEXT: EMPTY
+# BOLT-NEXT: ]
+# BOLT-NEXT: Bucket 6 [
+# BOLT-NEXT: EMPTY
+# BOLT-NEXT: ]
+# BOLT-NEXT: Bucket 7 [
+# BOLT-NEXT: Name 5 {
+# BOLT-NEXT: Hash: 0x65788E1C
+# BOLT-NEXT: String: {{.+}} "fully_specified"
+# BOLT-NEXT: Entry @ {{.+}} {
+# BOLT-NEXT: Abbrev: [[ABBREV1]]
+# BOLT-NEXT: Tag: DW_TAG_variable
+# BOLT-NEXT: DW_IDX_die_offset: 0x00000024
+# BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+# BOLT-NEXT: }
+# BOLT-NEXT: }
+# BOLT-NEXT: Name 6 {
+# BOLT-NEXT: Hash: 0x7C9A7F6A
+# BOLT-NEXT: String: {{.+}} "main"
+# BOLT-NEXT: Entry @ {{.+}} {
+# BOLT-NEXT: Abbrev: [[ABBREV4]]
+# BOLT-NEXT: Tag: DW_TAG_subprogram
+# BOLT-NEXT: DW_IDX_die_offset: 0x00000070
+# BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+# BOLT-NEXT: }
+# BOLT-NEXT: }
+# BOLT-NEXT: ]
+# BOLT-NEXT: Bucket 8 [
+# BOLT-NEXT: Name 7 {
+# BOLT-NEXT: Hash: 0xCEF4CFB
+# BOLT-NEXT: String: {{.+}} "__ARRAY_SIZE_TYPE__"
+# BOLT-NEXT: Entry @ {{.+}} {
+# BOLT-NEXT: Abbrev: [[ABBREV3]]
+# BOLT-NEXT: Tag: DW_TAG_base_type
+# BOLT-NEXT: DW_IDX_die_offset: 0x00000053
+# BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+# BOLT-NEXT: }
+# BOLT-NEXT: }
+# BOLT-NEXT: Name 8 {
+# BOLT-NEXT: Hash: 0x48684B69
+# BOLT-NEXT: String: {{.+}} "_ZN1A4smemE"
+# BOLT-NEXT: Entry @ {{.+}} {
+# BOLT-NEXT: Abbrev: [[ABBREV1]]
+# BOLT-NEXT: Tag: DW_TAG_variable
+# BOLT-NEXT: DW_IDX_die_offset: 0x00000057
+# BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+# BOLT-NEXT: }
+# BOLT-NEXT: }
+# BOLT-NEXT: Name 9 {
+# BOLT-NEXT: Hash: 0x7C952063
+# BOLT-NEXT: String: {{.+}} "char"
+# BOLT-NEXT: Entry @ {{.+}} {
+# BOLT-NEXT: Abbrev: [[ABBREV3]]
+# BOLT-NEXT: Tag: DW_TAG_base_type
+# BOLT-NEXT: DW_IDX_die_offset: 0x0000009e
+# BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+# BOLT-NEXT: }
+# BOLT-NEXT: }
+# BOLT-NEXT: ]
+# BOLT-NEXT: }
+
+# clang++ main.cpp -O2 -g2 -gdwarf-5 -gpubnames -S
+# struct A {
+# static int fully_specified;
+# static int smem[];
+# };
+#
+# int A::fully_specified;
+# int A::smem[] = { 0, 1, 2, 3 };
+# int main(int argc, char *argv[]) {
+# return 0;
+# }
+ .text
+ .file "main.cpp"
+ .file 0 "/specification" "main.cpp" md5 0x6c1b1c014d300f2e0efd26584acae1a9
+ .globl main # -- Begin function main
+ .p2align 4, 0x90
+ .type main, at function
+main: # @main
+.Lfunc_begin0:
+ .cfi_startproc
+# %bb.0: # %entry
+ #DEBUG_VALUE: main:argc <- $edi
+ #DEBUG_VALUE: main:argv <- $rsi
+ .loc 0 9 3 prologue_end # main.cpp:9:3
+ xorl %eax, %eax
+ retq
+.Ltmp0:
+.Lfunc_end0:
+ .size main, .Lfunc_end0-main
+ .cfi_endproc
+ # -- End function
+ .type _ZN1A15fully_specifiedE, at object # @_ZN1A15fully_specifiedE
+ .bss
+ .globl _ZN1A15fully_specifiedE
+ .p2align 2, 0x0
+_ZN1A15fully_specifiedE:
+ .long 0 # 0x0
+ .size _ZN1A15fully_specifiedE, 4
+
+ .type _ZN1A4smemE, at object # @_ZN1A4smemE
+ .data
+ .globl _ZN1A4smemE
+ .p2align 4, 0x0
+_ZN1A4smemE:
+ .long 0 # 0x0
+ .long 1 # 0x1
+ .long 2 # 0x2
+ .long 3 # 0x3
+ .size _ZN1A4smemE, 16
+
+ .section .debug_abbrev,"", at progbits
+ .byte 1 # Abbreviation Code
+ .byte 17 # DW_TAG_compile_unit
+ .byte 1 # DW_CHILDREN_yes
+ .byte 37 # DW_AT_producer
+ .byte 37 # DW_FORM_strx1
+ .byte 19 # DW_AT_language
+ .byte 5 # DW_FORM_data2
+ .byte 3 # DW_AT_name
+ .byte 37 # DW_FORM_strx1
+ .byte 114 # DW_AT_str_offsets_base
+ .byte 23 # DW_FORM_sec_offset
+ .byte 16 # DW_AT_stmt_list
+ .byte 23 # DW_FORM_sec_offset
+ .byte 27 # DW_AT_comp_dir
+ .byte 37 # DW_FORM_strx1
+ .byte 17 # DW_AT_low_pc
+ .byte 27 # DW_FORM_addrx
+ .byte 18 # DW_AT_high_pc
+ .byte 6 # DW_FORM_data4
+ .byte 115 # DW_AT_addr_base
+ .byte 23 # DW_FORM_sec_offset
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 2 # Abbreviation Code
+ .byte 52 # DW_TAG_variable
+ .byte 0 # DW_CHILDREN_no
+ .byte 71 # DW_AT_specification
+ .byte 19 # DW_FORM_ref4
+ .byte 2 # DW_AT_location
+ .byte 24 # DW_FORM_exprloc
+ .byte 110 # DW_AT_linkage_name
+ .byte 37 # DW_FORM_strx1
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 3 # Abbreviation Code
+ .byte 19 # DW_TAG_structure_type
+ .byte 1 # DW_CHILDREN_yes
+ .byte 54 # DW_AT_calling_convention
+ .byte 11 # DW_FORM_data1
+ .byte 3 # DW_AT_name
+ .byte 37 # DW_FORM_strx1
+ .byte 11 # DW_AT_byte_size
+ .byte 11 # DW_FORM_data1
+ .byte 58 # DW_AT_decl_file
+ .byte 11 # DW_FORM_data1
+ .byte 59 # DW_AT_decl_line
+ .byte 11 # DW_FORM_data1
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 4 # Abbreviation Code
+ .byte 52 # DW_TAG_variable
+ .byte 0 # DW_CHILDREN_no
+ .byte 3 # DW_AT_name
+ .byte 37 # DW_FORM_strx1
+ .byte 73 # DW_AT_type
+ .byte 19 # DW_FORM_ref4
+ .byte 58 # DW_AT_decl_file
+ .byte 11 # DW_FORM_data1
+ .byte 59 # DW_AT_decl_line
+ .byte 11 # DW_FORM_data1
+ .byte 63 # DW_AT_external
+ .byte 25 # DW_FORM_flag_present
+ .byte 60 # DW_AT_declaration
+ .byte 25 # DW_FORM_flag_present
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 5 # Abbreviation Code
+ .byte 36 # DW_TAG_base_type
+ .byte 0 # DW_CHILDREN_no
+ .byte 3 # DW_AT_name
+ .byte 37 # DW_FORM_strx1
+ .byte 62 # DW_AT_encoding
+ .byte 11 # DW_FORM_data1
+ .byte 11 # DW_AT_byte_size
+ .byte 11 # DW_FORM_data1
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 6 # Abbreviation Code
+ .byte 1 # DW_TAG_array_type
+ .byte 1 # DW_CHILDREN_yes
+ .byte 73 # DW_AT_type
+ .byte 19 # DW_FORM_ref4
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 7 # Abbreviation Code
+ .byte 33 # DW_TAG_subrange_type
+ .byte 0 # DW_CHILDREN_no
+ .byte 73 # DW_AT_type
+ .byte 19 # DW_FORM_ref4
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 8 # Abbreviation Code
+ .byte 36 # DW_TAG_base_type
+ .byte 0 # DW_CHILDREN_no
+ .byte 3 # DW_AT_name
+ .byte 37 # DW_FORM_strx1
+ .byte 11 # DW_AT_byte_size
+ .byte 11 # DW_FORM_data1
+ .byte 62 # DW_AT_encoding
+ .byte 11 # DW_FORM_data1
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 9 # Abbreviation Code
+ .byte 52 # DW_TAG_variable
+ .byte 0 # DW_CHILDREN_no
+ .byte 71 # DW_AT_specification
+ .byte 19 # DW_FORM_ref4
+ .byte 73 # DW_AT_type
+ .byte 19 # DW_FORM_ref4
+ .byte 2 # DW_AT_location
+ .byte 24 # DW_FORM_exprloc
+ .byte 110 # DW_AT_linkage_name
+ .byte 37 # DW_FORM_strx1
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 10 # Abbreviation Code
+ .byte 33 # DW_TAG_subrange_type
+ .byte 0 # DW_CHILDREN_no
+ .byte 73 # DW_AT_type
+ .byte 19 # DW_FORM_ref4
+ .byte 55 # DW_AT_count
+ .byte 11 # DW_FORM_data1
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 11 # Abbreviation Code
+ .byte 46 # DW_TAG_subprogram
+ .byte 1 # DW_CHILDREN_yes
+ .byte 17 # DW_AT_low_pc
+ .byte 27 # DW_FORM_addrx
+ .byte 18 # DW_AT_high_pc
+ .byte 6 # DW_FORM_data4
+ .byte 64 # DW_AT_frame_base
+ .byte 24 # DW_FORM_exprloc
+ .byte 122 # DW_AT_call_all_calls
+ .byte 25 # DW_FORM_flag_present
+ .byte 3 # DW_AT_name
+ .byte 37 # DW_FORM_strx1
+ .byte 58 # DW_AT_decl_file
+ .byte 11 # DW_FORM_data1
+ .byte 59 # DW_AT_decl_line
+ .byte 11 # DW_FORM_data1
+ .byte 73 # DW_AT_type
+ .byte 19 # DW_FORM_ref4
+ .byte 63 # DW_AT_external
+ .byte 25 # DW_FORM_flag_present
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 12 # Abbreviation Code
+ .byte 5 # DW_TAG_formal_parameter
+ .byte 0 # DW_CHILDREN_no
+ .byte 2 # DW_AT_location
+ .byte 24 # DW_FORM_exprloc
+ .byte 3 # DW_AT_name
+ .byte 37 # DW_FORM_strx1
+ .byte 58 # DW_AT_decl_file
+ .byte 11 # DW_FORM_data1
+ .byte 59 # DW_AT_decl_line
+ .byte 11 # DW_FORM_data1
+ .byte 73 # DW_AT_type
+ .byte 19 # DW_FORM_ref4
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 13 # Abbreviation Code
+ .byte 15 # DW_TAG_pointer_type
+ .byte 0 # DW_CHILDREN_no
+ .byte 73 # DW_AT_type
+ .byte 19 # DW_FORM_ref4
+ .byte 0 # EOM(1)
+ .byte 0 # EOM(2)
+ .byte 0 # EOM(3)
+ .section .debug_info,"", at progbits
+.Lcu_begin0:
+ .long .Ldebug_info_end0-.Ldebug_info_start0 # Length of Unit
+.Ldebug_info_start0:
+ .short 5 # DWARF version number
+ .byte 1 # DWARF Unit Type
+ .byte 8 # Address Size (in bytes)
+ .long .debug_abbrev # Offset Into Abbrev. Section
+ .byte 1 # Abbrev [1] 0xc:0x96 DW_TAG_compile_unit
+ .byte 0 # DW_AT_producer
+ .short 33 # DW_AT_language
+ .byte 1 # DW_AT_name
+ .long .Lstr_offsets_base0 # DW_AT_str_offsets_base
+ .long .Lline_table_start0 # DW_AT_stmt_list
+ .byte 2 # DW_AT_comp_dir
+ .byte 2 # DW_AT_low_pc
+ .long .Lfunc_end0-.Lfunc_begin0 # DW_AT_high_pc
+ .long .Laddr_table_base0 # DW_AT_addr_base
+ .byte 2 # Abbrev [2] 0x23:0x9 DW_TAG_variable
+ .long 50 # DW_AT_specification
+ .byte 2 # DW_AT_location
+ .byte 161
+ .byte 0
+ .byte 8 # DW_AT_linkage_name
+ .byte 3 # Abbrev [3] 0x2c:0x17 DW_TAG_structure_type
+ .byte 5 # DW_AT_calling_convention
+ .byte 7 # DW_AT_name
+ .byte 1 # DW_AT_byte_size
+ .byte 0 # DW_AT_decl_file
+ .byte 1 # DW_AT_decl_line
+ .byte 4 # Abbrev [4] 0x32:0x8 DW_TAG_variable
+ .byte 3 # DW_AT_name
+ .long 67 # DW_AT_type
+ .byte 0 # DW_AT_decl_file
+ .byte 2 # DW_AT_decl_line
+ # DW_AT_external
+ # DW_AT_declaration
+ .byte 4 # Abbrev [4] 0x3a:0x8 DW_TAG_variable
+ .byte 5 # DW_AT_name
+ .long 71 # DW_AT_type
+ .byte 0 # DW_AT_decl_file
+ .byte 3 # DW_AT_decl_line
+ # DW_AT_external
+ # DW_AT_declaration
+ .byte 0 # End Of Children Mark
+ .byte 5 # Abbrev [5] 0x43:0x4 DW_TAG_base_type
+ .byte 4 # DW_AT_name
+ .byte 5 # DW_AT_encoding
+ .byte 4 # DW_AT_byte_size
+ .byte 6 # Abbrev [6] 0x47:0xb DW_TAG_array_type
+ .long 67 # DW_AT_type
+ .byte 7 # Abbrev [7] 0x4c:0x5 DW_TAG_subrange_type
+ .long 82 # DW_AT_type
+ .byte 0 # End Of Children Mark
+ .byte 8 # Abbrev [8] 0x52:0x4 DW_TAG_base_type
+ .byte 6 # DW_AT_name
+ .byte 8 # DW_AT_byte_size
+ .byte 7 # DW_AT_encoding
+ .byte 9 # Abbrev [9] 0x56:0xd DW_TAG_variable
+ .long 58 # DW_AT_specification
+ .long 99 # DW_AT_type
+ .byte 2 # DW_AT_location
+ .byte 161
+ .byte 1
+ .byte 9 # DW_AT_linkage_name
+ .byte 6 # Abbrev [6] 0x63:0xc DW_TAG_array_type
+ .long 67 # DW_AT_type
+ .byte 10 # Abbrev [10] 0x68:0x6 DW_TAG_subrange_type
+ .long 82 # DW_AT_type
+ .byte 4 # DW_AT_count
+ .byte 0 # End Of Children Mark
+ .byte 11 # Abbrev [11] 0x6f:0x24 DW_TAG_subprogram
+ .byte 2 # DW_AT_low_pc
+ .long .Lfunc_end0-.Lfunc_begin0 # DW_AT_high_pc
+ .byte 1 # DW_AT_frame_base
+ .byte 87
+ # DW_AT_call_all_calls
+ .byte 10 # DW_AT_name
+ .byte 0 # DW_AT_decl_file
+ .byte 8 # DW_AT_decl_line
+ .long 67 # DW_AT_type
+ # DW_AT_external
+ .byte 12 # Abbrev [12] 0x7e:0xa DW_TAG_formal_parameter
+ .byte 1 # DW_AT_location
+ .byte 85
+ .byte 11 # DW_AT_name
+ .byte 0 # DW_AT_decl_file
+ .byte 8 # DW_AT_decl_line
+ .long 67 # DW_AT_type
+ .byte 12 # Abbrev [12] 0x88:0xa DW_TAG_formal_parameter
+ .byte 1 # DW_AT_location
+ .byte 84
+ .byte 12 # DW_AT_name
+ .byte 0 # DW_AT_decl_file
+ .byte 8 # DW_AT_decl_line
+ .long 147 # DW_AT_type
+ .byte 0 # End Of Children Mark
+ .byte 13 # Abbrev [13] 0x93:0x5 DW_TAG_pointer_type
+ .long 152 # DW_AT_type
+ .byte 13 # Abbrev [13] 0x98:0x5 DW_TAG_pointer_type
+ .long 157 # DW_AT_type
+ .byte 5 # Abbrev [5] 0x9d:0x4 DW_TAG_base_type
+ .byte 13 # DW_AT_name
+ .byte 6 # DW_AT_encoding
+ .byte 1 # DW_AT_byte_size
+ .byte 0 # End Of Children Mark
+.Ldebug_info_end0:
+ .section .debug_str_offsets,"", at progbits
+ .long 60 # Length of String Offsets Set
+ .short 5
+ .short 0
+.Lstr_offsets_base0:
+ .section .debug_str,"MS", at progbits,1
+.Linfo_string0:
+ .asciz "clang version 19.0.0git (git at github.com:llvm/llvm-project.git ced1fac8a32e35b63733bda27c7f5b9a2b635403)" # string offset=0
+.Linfo_string1:
+ .asciz "main.cpp" # string offset=104
+.Linfo_string2:
+ .asciz "/specification" # string offset=113
+.Linfo_string3:
+ .asciz "A" # string offset=165
+.Linfo_string4:
+ .asciz "fully_specified" # string offset=167
+.Linfo_string5:
+ .asciz "int" # string offset=183
+.Linfo_string6:
+ .asciz "smem" # string offset=187
+.Linfo_string7:
+ .asciz "__ARRAY_SIZE_TYPE__" # string offset=192
+.Linfo_string8:
+ .asciz "_ZN1A15fully_specifiedE" # string offset=212
+.Linfo_string9:
+ .asciz "_ZN1A4smemE" # string offset=236
+.Linfo_string10:
+ .asciz "main" # string offset=248
+.Linfo_string11:
+ .asciz "argc" # string offset=253
+.Linfo_string12:
+ .asciz "argv" # string offset=258
+.Linfo_string13:
+ .asciz "char" # string offset=263
+ .section .debug_str_offsets,"", at progbits
+ .long .Linfo_string0
+ .long .Linfo_string1
+ .long .Linfo_string2
+ .long .Linfo_string4
+ .long .Linfo_string5
+ .long .Linfo_string6
+ .long .Linfo_string7
+ .long .Linfo_string3
+ .long .Linfo_string8
+ .long .Linfo_string9
+ .long .Linfo_string10
+ .long .Linfo_string11
+ .long .Linfo_string12
+ .long .Linfo_string13
+ .section .debug_addr,"", at progbits
+ .long .Ldebug_addr_end0-.Ldebug_addr_start0 # Length of contribution
+.Ldebug_addr_start0:
+ .short 5 # DWARF version number
+ .byte 8 # Address size
+ .byte 0 # Segment selector size
+.Laddr_table_base0:
+ .quad _ZN1A15fully_specifiedE
+ .quad _ZN1A4smemE
+ .quad .Lfunc_begin0
+.Ldebug_addr_end0:
+ .section .debug_names,"", at progbits
+ .long .Lnames_end0-.Lnames_start0 # Header: unit length
+.Lnames_start0:
+ .short 5 # Header: version
+ .short 0 # Header: padding
+ .long 1 # Header: compilation unit count
+ .long 0 # Header: local type unit count
+ .long 0 # Header: foreign type unit count
+ .long 9 # Header: bucket count
+ .long 9 # Header: name count
+ .long .Lnames_abbrev_end0-.Lnames_abbrev_start0 # Header: abbreviation table size
+ .long 8 # Header: augmentation string size
+ .ascii "LLVM0700" # Header: augmentation string
+ .long .Lcu_begin0 # Compilation unit 0
+ .long 1 # Bucket 0
+ .long 3 # Bucket 1
+ .long 4 # Bucket 2
+ .long 0 # Bucket 3
+ .long 0 # Bucket 4
+ .long 0 # Bucket 5
+ .long 0 # Bucket 6
+ .long 5 # Bucket 7
+ .long 7 # Bucket 8
+ .long 1564256736 # Hash in Bucket 0
+ .long 2090728503 # Hash in Bucket 0
+ .long 177670 # Hash in Bucket 1
+ .long 193495088 # Hash in Bucket 2
+ .long 1702399516 # Hash in Bucket 7
+ .long 2090499946 # Hash in Bucket 7
+ .long 217009403 # Hash in Bucket 8
+ .long 1214794601 # Hash in Bucket 8
+ .long 2090147939 # Hash in Bucket 8
+ .long .Linfo_string8 # String in Bucket 0: _ZN1A15fully_specifiedE
+ .long .Linfo_string6 # String in Bucket 0: smem
+ .long .Linfo_string3 # String in Bucket 1: A
+ .long .Linfo_string5 # String in Bucket 2: int
+ .long .Linfo_string4 # String in Bucket 7: fully_specified
+ .long .Linfo_string10 # String in Bucket 7: main
+ .long .Linfo_string7 # String in Bucket 8: __ARRAY_SIZE_TYPE__
+ .long .Linfo_string9 # String in Bucket 8: _ZN1A4smemE
+ .long .Linfo_string13 # String in Bucket 8: char
+ .long .Lnames4-.Lnames_entries0 # Offset in Bucket 0
+ .long .Lnames5-.Lnames_entries0 # Offset in Bucket 0
+ .long .Lnames0-.Lnames_entries0 # Offset in Bucket 1
+ .long .Lnames1-.Lnames_entries0 # Offset in Bucket 2
+ .long .Lnames3-.Lnames_entries0 # Offset in Bucket 7
+ .long .Lnames7-.Lnames_entries0 # Offset in Bucket 7
+ .long .Lnames2-.Lnames_entries0 # Offset in Bucket 8
+ .long .Lnames6-.Lnames_entries0 # Offset in Bucket 8
+ .long .Lnames8-.Lnames_entries0 # Offset in Bucket 8
+.Lnames_abbrev_start0:
+ .byte 1 # Abbrev code
+ .byte 52 # DW_TAG_variable
+ .byte 3 # DW_IDX_die_offset
+ .byte 19 # DW_FORM_ref4
+ .byte 4 # DW_IDX_parent
+ .byte 25 # DW_FORM_flag_present
+ .byte 0 # End of abbrev
+ .byte 0 # End of abbrev
+ .byte 2 # Abbrev code
+ .byte 19 # DW_TAG_structure_type
+ .byte 3 # DW_IDX_die_offset
+ .byte 19 # DW_FORM_ref4
+ .byte 4 # DW_IDX_parent
+ .byte 25 # DW_FORM_flag_present
+ .byte 0 # End of abbrev
+ .byte 0 # End of abbrev
+ .byte 3 # Abbrev code
+ .byte 36 # DW_TAG_base_type
+ .byte 3 # DW_IDX_die_offset
+ .byte 19 # DW_FORM_ref4
+ .byte 4 # DW_IDX_parent
+ .byte 25 # DW_FORM_flag_present
+ .byte 0 # End of abbrev
+ .byte 0 # End of abbrev
+ .byte 4 # Abbrev code
+ .byte 46 # DW_TAG_subprogram
+ .byte 3 # DW_IDX_die_offset
+ .byte 19 # DW_FORM_ref4
+ .byte 4 # DW_IDX_parent
+ .byte 25 # DW_FORM_flag_present
+ .byte 0 # End of abbrev
+ .byte 0 # End of abbrev
+ .byte 0 # End of abbrev list
+.Lnames_abbrev_end0:
+.Lnames_entries0:
+.Lnames4:
+.L3:
+ .byte 1 # Abbreviation code
+ .long 35 # DW_IDX_die_offset
+ .byte 0 # DW_IDX_parent
+ # End of list: _ZN1A15fully_specifiedE
+.Lnames5:
+.L4:
+ .byte 1 # Abbreviation code
+ .long 86 # DW_IDX_die_offset
+ .byte 0 # DW_IDX_parent
+ # End of list: smem
+.Lnames0:
+.L6:
+ .byte 2 # Abbreviation code
+ .long 44 # DW_IDX_die_offset
+ .byte 0 # DW_IDX_parent
+ # End of list: A
+.Lnames1:
+.L5:
+ .byte 3 # Abbreviation code
+ .long 67 # DW_IDX_die_offset
+ .byte 0 # DW_IDX_parent
+ # End of list: int
+.Lnames3:
+ .byte 1 # Abbreviation code
+ .long 35 # DW_IDX_die_offset
+ .byte 0 # DW_IDX_parent
+ # End of list: fully_specified
+.Lnames7:
+.L0:
+ .byte 4 # Abbreviation code
+ .long 111 # DW_IDX_die_offset
+ .byte 0 # DW_IDX_parent
+ # End of list: main
+.Lnames2:
+.L2:
+ .byte 3 # Abbreviation code
+ .long 82 # DW_IDX_die_offset
+ .byte 0 # DW_IDX_parent
+ # End of list: __ARRAY_SIZE_TYPE__
+.Lnames6:
+ .byte 1 # Abbreviation code
+ .long 86 # DW_IDX_die_offset
+ .byte 0 # DW_IDX_parent
+ # End of list: _ZN1A4smemE
+.Lnames8:
+.L1:
+ .byte 3 # Abbreviation code
+ .long 157 # DW_IDX_die_offset
+ .byte 0 # DW_IDX_parent
+ # End of list: char
+ .p2align 2, 0x0
+.Lnames_end0:
+ .ident "clang version 19.0.0git (git at github.com:llvm/llvm-project.git ced1fac8a32e35b63733bda27c7f5b9a2b635403)"
+ .section ".note.GNU-stack","", at progbits
+ .addrsig
+ .section .debug_line,"", at progbits
+.Lline_table_start0:
diff --git a/bolt/test/X86/dwarf5-debug-names-generate-debug-names.test b/bolt/test/X86/dwarf5-debug-names-generate-debug-names.test
index ae921d1e8b93bc..b7d8fda63315fa 100644
--- a/bolt/test/X86/dwarf5-debug-names-generate-debug-names.test
+++ b/bolt/test/X86/dwarf5-debug-names-generate-debug-names.test
@@ -14,15 +14,15 @@
; BOLT: [[OFFSET2:0x[0-9a-f]*]]: Compile Unit
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0x109
+; BOLT-NEXT: Length: 0x155
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 2
; BOLT-NEXT: Local TU count: 0
; BOLT-NEXT: Foreign TU count: 0
-; BOLT-NEXT: Bucket count: 8
-; BOLT-NEXT: Name count: 8
-; BOLT-NEXT: Abbreviations table size: 0x1F
+; BOLT-NEXT: Bucket count: 10
+; BOLT-NEXT: Name count: 10
+; BOLT-NEXT: Abbreviations table size: 0x29
; BOLT-NEXT: Augmentation: 'BOLT'
; BOLT-NEXT: }
; BOLT-NEXT: Compilation Unit offsets [
@@ -31,136 +31,183 @@
; BOLT-NEXT: ]
; BOLT-NEXT: Abbreviations [
; BOLT-NEXT: Abbreviation [[ABBREV1:0x[0-9a-f]*]] {
-; BOLT-NEXT: Tag: DW_TAG_base_type
+; BOLT-NEXT: Tag: DW_TAG_subprogram
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: Abbreviation [[ABBREV2:0x[0-9a-f]*]] {
-; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: Tag: DW_TAG_inlined_subroutine
+; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
+; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
+; BOLT-NEXT: DW_IDX_parent: DW_FORM_ref4
+; BOLT-NEXT: }
+; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
+; BOLT-NEXT: Tag: DW_TAG_variable
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
-; BOLT-NEXT: Abbreviation [[ABBREV3:0x[0-9a-f]*]] {
-; BOLT-NEXT: Tag: DW_TAG_variable
+; BOLT-NEXT: Abbreviation [[ABBREV4:0x[0-9a-f]*]] {
+; BOLT-NEXT: Tag: DW_TAG_base_type
; BOLT-NEXT: DW_IDX_compile_unit: DW_FORM_data1
; BOLT-NEXT: DW_IDX_die_offset: DW_FORM_ref4
; BOLT-NEXT: DW_IDX_parent: DW_FORM_flag_present
; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 0 [
-; BOLT-NEXT: Name 1 {
-; BOLT-NEXT: Hash: 0xB888030
-; BOLT-NEXT: String: {{.+}} "int"
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV1]]
-; BOLT-NEXT: Tag: DW_TAG_base_type
-; BOLT-NEXT: DW_IDX_compile_unit: 0x01
-; BOLT-NEXT: DW_IDX_die_offset: 0x00000033
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: Name 1 {
+; BOLT-NEXT: Hash: 0x2124BB36
+; BOLT-NEXT: String: {{.+}} "useFoo"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000037
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV2]]
+; BOLT-NEXT: Tag: DW_TAG_inlined_subroutine
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000081
+; BOLT-NEXT: DW_IDX_parent: Entry @ 0x14b
+; BOLT-NEXT: }
; BOLT-NEXT: }
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV1]]
-; BOLT-NEXT: Tag: DW_TAG_base_type
-; BOLT-NEXT: DW_IDX_compile_unit: 0x00
-; BOLT-NEXT: DW_IDX_die_offset: 0x0000007f
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 1 [
+; BOLT-NEXT: EMPTY
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 2 [
+; BOLT-NEXT: Name 2 {
+; BOLT-NEXT: Hash: 0xFDE4B5D2
+; BOLT-NEXT: String: {{.+}} "fooVar"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV3]]
+; BOLT-NEXT: Tag: DW_TAG_variable
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000028
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
; BOLT-NEXT: }
-; BOLT-NEXT: }
-; BOLT-NEXT: ]
-; BOLT-NEXT: Bucket 1 [
-; BOLT-NEXT: Name 2 {
-; BOLT-NEXT: Hash: 0xB887389
-; BOLT-NEXT: String: {{.+}} "foo"
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV2]]
-; BOLT-NEXT: Tag: DW_TAG_subprogram
-; BOLT-NEXT: DW_IDX_compile_unit: 0x01
-; BOLT-NEXT: DW_IDX_die_offset: 0x0000005e
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 3 [
+; BOLT-NEXT: EMPTY
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 4 [
+; BOLT-NEXT: EMPTY
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 5 [
+; BOLT-NEXT: EMPTY
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 6 [
+; BOLT-NEXT: Name 3 {
+; BOLT-NEXT: Hash: 0xB88B3D2
+; BOLT-NEXT: String: {{.+}} "use"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000028
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
; BOLT-NEXT: }
-; BOLT-NEXT: }
-; BOLT-NEXT: Name 3 {
-; BOLT-NEXT: Hash: 0x8C06E589
-; BOLT-NEXT: String: {{.+}} "_Z3usePiS_"
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV2]]
-; BOLT-NEXT: Tag: DW_TAG_subprogram
-; BOLT-NEXT: DW_IDX_compile_unit: 0x00
-; BOLT-NEXT: DW_IDX_die_offset: 0x00000028
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: Name 4 {
+; BOLT-NEXT: Hash: 0x69A7307E
+; BOLT-NEXT: String: {{.+}} "_Z6useFooPi"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000037
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV2]]
+; BOLT-NEXT: Tag: DW_TAG_inlined_subroutine
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000081
+; BOLT-NEXT: DW_IDX_parent: Entry @ 0x14b
+; BOLT-NEXT: }
; BOLT-NEXT: }
-; BOLT-NEXT: }
-; BOLT-NEXT: ]
-; BOLT-NEXT: Bucket 2 [
-; BOLT-NEXT: Name 4 {
-; BOLT-NEXT: Hash: 0xB88B3D2
-; BOLT-NEXT: String: {{.+}} "use"
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV2]]
-; BOLT-NEXT: Tag: DW_TAG_subprogram
-; BOLT-NEXT: DW_IDX_compile_unit: 0x00
-; BOLT-NEXT: DW_IDX_die_offset: 0x00000028
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: Name 5 {
+; BOLT-NEXT: Hash: 0x7C9A7F6A
+; BOLT-NEXT: String: {{.+}} "main"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000049
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
; BOLT-NEXT: }
-; BOLT-NEXT: }
-; BOLT-NEXT: Name 5 {
-; BOLT-NEXT: Hash: 0x7C9A7F6A
-; BOLT-NEXT: String: {{.+}} "main"
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV2]]
-; BOLT-NEXT: Tag: DW_TAG_subprogram
-; BOLT-NEXT: DW_IDX_compile_unit: 0x00
-; BOLT-NEXT: DW_IDX_die_offset: 0x00000049
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: Name 6 {
+; BOLT-NEXT: Hash: 0xB5063CFE
+; BOLT-NEXT: String: {{.+}} "_Z3fooi"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x0000005e
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
; BOLT-NEXT: }
-; BOLT-NEXT: }
-; BOLT-NEXT: Name 6 {
-; BOLT-NEXT: Hash: 0xFDE4B5D2
-; BOLT-NEXT: String: {{.+}} "fooVar"
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV3]]
-; BOLT-NEXT: Tag: DW_TAG_variable
-; BOLT-NEXT: DW_IDX_compile_unit: 0x01
-; BOLT-NEXT: DW_IDX_die_offset: 0x00000028
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 7 [
+; BOLT-NEXT: Name 7 {
+; BOLT-NEXT: Hash: 0x8C06E589
+; BOLT-NEXT: String: {{.+}} "_Z3usePiS_"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000028
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
; BOLT-NEXT: }
-; BOLT-NEXT: }
-; BOLT-NEXT: ]
-; BOLT-NEXT: Bucket 3 [
-; BOLT-NEXT: Name 7 {
-; BOLT-NEXT: Hash: 0x7C952063
-; BOLT-NEXT: String: {{.+}} "char"
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV1]]
-; BOLT-NEXT: Tag: DW_TAG_base_type
-; BOLT-NEXT: DW_IDX_compile_unit: 0x00
-; BOLT-NEXT: DW_IDX_die_offset: 0x00000092
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 8 [
+; BOLT-NEXT: Name 8 {
+; BOLT-NEXT: Hash: 0xB888030
+; BOLT-NEXT: String: {{.+}} "int"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV4]]
+; BOLT-NEXT: Tag: DW_TAG_base_type
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000033
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: Entry @ 0x{{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV4]]
+; BOLT-NEXT: Tag: DW_TAG_base_type
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x0000007f
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
; BOLT-NEXT: }
-; BOLT-NEXT: }
-; BOLT-NEXT: ]
-; BOLT-NEXT: Bucket 4 [
-; BOLT-NEXT: EMPTY
-; BOLT-NEXT: ]
-; BOLT-NEXT: Bucket 5 [
-; BOLT-NEXT: EMPTY
-; BOLT-NEXT: ]
-; BOLT-NEXT: Bucket 6 [
-; BOLT-NEXT: Name 8 {
-; BOLT-NEXT: Hash: 0xB5063CFE
-; BOLT-NEXT: String: {{.+}} "_Z3fooi"
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV2]]
-; BOLT-NEXT: Tag: DW_TAG_subprogram
-; BOLT-NEXT: DW_IDX_compile_unit: 0x01
-; BOLT-NEXT: DW_IDX_die_offset: 0x0000005e
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: ]
+; BOLT-NEXT: Bucket 9 [
+; BOLT-NEXT: Name 9 {
+; BOLT-NEXT: Hash: 0xB887389
+; BOLT-NEXT: String: {{.+}} "foo"
+; BOLT-NEXT: Entry @ 0x14b {
+; BOLT-NEXT: Abbrev: [[ABBREV1]]
+; BOLT-NEXT: Tag: DW_TAG_subprogram
+; BOLT-NEXT: DW_IDX_compile_unit: 0x01
+; BOLT-NEXT: DW_IDX_die_offset: 0x0000005e
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
; BOLT-NEXT: }
-; BOLT-NEXT: }
-; BOLT-NEXT: ]
-; BOLT-NEXT: Bucket 7 [
-; BOLT-NEXT: EMPTY
-; BOLT-NEXT: ]
-; BOLT-NEXT: }
+; BOLT-NEXT: Name 10 {
+; BOLT-NEXT: Hash: 0x7C952063
+; BOLT-NEXT: String: {{.+}} "char"
+; BOLT-NEXT: Entry @ {{.+}} {
+; BOLT-NEXT: Abbrev: [[ABBREV4]]
+; BOLT-NEXT: Tag: DW_TAG_base_type
+; BOLT-NEXT: DW_IDX_compile_unit: 0x00
+; BOLT-NEXT: DW_IDX_die_offset: 0x00000092
+; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
+; BOLT-NEXT: }
+; BOLT-NEXT: }
+; BOLT-NEXT: ]
+; BOLT-NEXT: }
diff --git a/bolt/test/X86/dwarf5-debug-names.test b/bolt/test/X86/dwarf5-debug-names.test
index 8e45ef3a948123..ef11ee0c479b7f 100644
--- a/bolt/test/X86/dwarf5-debug-names.test
+++ b/bolt/test/X86/dwarf5-debug-names.test
@@ -11,7 +11,7 @@
; BOLT: [[OFFSET2:0x[0-9a-f]*]]: Compile Unit
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0x1DA
+; BOLT-NEXT: Length: 0x1D4
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 2
@@ -101,13 +101,6 @@
; BOLT-NEXT: DW_IDX_die_offset: 0x00000061
; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV2]]
-; BOLT-NEXT: Tag: DW_TAG_namespace
-; BOLT-NEXT: DW_IDX_compile_unit: 0x00
-; BOLT-NEXT: DW_IDX_die_offset: 0x00000061
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
-; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: Name 4 {
; BOLT-NEXT: Hash: 0xBA564846
diff --git a/bolt/test/X86/dwarf5-one-cu-debug-names.test b/bolt/test/X86/dwarf5-one-cu-debug-names.test
index 2145d8160a42dc..74d6b3041c9335 100644
--- a/bolt/test/X86/dwarf5-one-cu-debug-names.test
+++ b/bolt/test/X86/dwarf5-one-cu-debug-names.test
@@ -9,7 +9,7 @@
; BOLT: [[OFFSET1:0x[0-9a-f]*]]: Compile Unit
; BOLT: Name Index @ 0x0 {
; BOLT-NEXT: Header {
-; BOLT-NEXT: Length: 0x154
+; BOLT-NEXT: Length: 0x14F
; BOLT-NEXT: Format: DWARF32
; BOLT-NEXT: Version: 5
; BOLT-NEXT: CU count: 1
@@ -183,12 +183,6 @@
; BOLT-NEXT: DW_IDX_die_offset: 0x00000061
; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
; BOLT-NEXT: }
-; BOLT-NEXT: Entry @ {{.+}} {
-; BOLT-NEXT: Abbrev: [[ABBREV6]]
-; BOLT-NEXT: Tag: DW_TAG_namespace
-; BOLT-NEXT: DW_IDX_die_offset: 0x00000061
-; BOLT-NEXT: DW_IDX_parent: <parent not indexed>
-; BOLT-NEXT: }
; BOLT-NEXT: }
; BOLT-NEXT: ]
; BOLT-NEXT: Bucket 9 [
More information about the llvm-commits
mailing list
diff --git a/bolt/include/bolt/Core/DebugNames.h b/bolt/include/bolt/Core/DebugNames.h
index 84c448aa1354cc..1f17f1ae4d139b 100644
--- a/bolt/include/bolt/Core/DebugNames.h
+++ b/bolt/include/bolt/Core/DebugNames.h
@@ -36,6 +36,9 @@ class BOLTDWARF5AccelTableData : public DWARF5AccelTableData {
bool isTU() const { return DWARF5AccelTableData::isTU(); }
std::optional<unsigned> getSecondUnitID() const { return SecondUnitID; }
+ void setPatchOffset(uint64_t PatchOffset) { OffsetVal = PatchOffset; }
+ uint64_t getPatchOffset() const { return std::get<uint64_t>(OffsetVal); }
+
private:
std::optional<unsigned> SecondUnitID;
};
@@ -49,10 +52,12 @@ class DWARF5AcceleratorTable {
Abbrev->~DebugNamesAbbrev();
}
/// Add DWARF5 Accelerator table entry.
- /// Input is DWARFUnit being processed, DIE that belongs to it, and potential
- /// SkeletonCU if the Unit comes from a DWO section.
- void addAccelTableEntry(DWARFUnit &Unit, const DIE &Die,
- const std::optional<uint64_t> &DWOID);
+ /// Input is DWARFUnit being processed, DIE that belongs to it, potential
+ /// DWOID if the Unit comes from a DWO section, and potential parent entry.
+ std::optional<BOLTDWARF5AccelTableData *>
+ addAccelTableEntry(DWARFUnit &Unit, const DIE &Die,
+ const std::optional<uint64_t> &DWOID,
+ std::optional<BOLTDWARF5AccelTableData *> &Parent);
/// Set current unit being processed.
void setCurrentUnit(DWARFUnit &Unit, const uint64_t UnitStartOffset);
/// Emit Accelerator table.
@@ -121,6 +126,8 @@ class DWARF5AcceleratorTable {
llvm::DenseMap<llvm::hash_code, uint64_t> StrCacheToOffsetMap;
// Contains DWO ID to CUList Index.
llvm::DenseMap<uint64_t, uint32_t> CUOffsetsToPatch;
+ // Contains a map of Entry ID to Entry relative offset.
+ llvm::DenseMap<uint64_t, uint32_t> EntryRelativeOffsets;
/// Adds Unit to either CUList, LocalTUList or ForeignTUList.
/// Input Unit being processed, and DWO ID if Unit is being processed comes
/// from a DWO section.
@@ -143,7 +150,7 @@ class DWARF5AcceleratorTable {
/// Write Entries.
void writeEntries();
/// Write an Entry.
- void writeEntry(const BOLTDWARF5AccelTableData &Entry);
+ void writeEntry(BOLTDWARF5AccelTableData &Entry);
/// Write augmentation_string for BOLT.
void writeAugmentationString();
/// Emit out Header for DWARF5 Accelerator table.